@@ -11,7 +11,7 @@ class EmbedGenerator(commands.Cog):
         self.color = 0x34b7eb
 
     @commands.has_permissions(manage_messages = True)
-    @commands.bot_has_permissions(manage_messages = True)
+    @commands.guild_only()
     @commands.command(
         name='createembed',
         brief='Erstelle einen Embed',
@@ -20,8 +20,8 @@ class EmbedGenerator(commands.Cog):
         help="Benutze /createembed für Erklärungen.",
         usage="<Titel> [Argumente]"
     )
-    async def createembed(self, ctx, *args):
-        text = " ".join(args)
+    async def createembed(self, ctx):
+        text = " ".join(ctx.message.content.split(ctx.invoked_with)[1:])
         if not text:
             await ctx.sendEmbed(title="Embed-Creator", description=f"""
 Verwendung des Chat-Embed-Generators:
@@ -34,7 +34,7 @@ Verwendung des Chat-Embed-Generators:
 Parameter brauchen je eine Zeile und beginnen immer mit `{par}`.
 Optionen werden immer mit `{opt}` getrennt.
 
-Verfügbare Parameter:""", color=self.color, inline=False, fields=[
+Verfügbare Parameter:""", inline=False, fields=[
         ("Feld (bis zu 25)",f"{par}field{opt}<Titel>{opt}<Inhalt>{opt}[Inline]"),
         ("Footer",          f"{par}footer{opt}<Titel>{opt}[Bild-URL]"),
         ("Author",          f"{par}author{opt}<Name>{opt}[Link]"),
@@ -46,7 +46,6 @@ Verfügbare Parameter:""", color=self.color, inline=False, fields=[
             data = {
                 "title": lines.pop(0),
                 "description": "",
-                "color": self.color,
                 "footertext": "",
                 "footerurl": "",
                 "authorname": "",
@@ -106,7 +105,7 @@ Verfügbare Parameter:""", color=self.color, inline=False, fields=[
                     text += f"\n{par}field{opt}{field.name}{opt}{field.value}{(opt+'False') if not field.inline else ''}"
             if emb.color:
                 text += f"\n{par}color{opt}{str(emb.color.r)}{opt}{str(emb.color.g)}{opt}{str(emb.color.b)}"
-            await ctx.sendEmbed(title="Embed Command", color=self.color, description=text)
+            await ctx.sendEmbed(title="Embed Command", description=text)
         else:
             raise commands.BadArgument("Diese Nachricht hat kein Embed!")
 
