@@ -3,7 +3,7 @@
 from asgiref.sync import sync_to_async
 
 from discord.ext import commands, tasks
-from discord import Embed, utils, PermissionOverwrite, Color, NotFound, User
+from discord import Embed, utils, PermissionOverwrite, Color, NotFound, User, Member
 
 from django.utils import timezone
 
@@ -11,10 +11,12 @@ from datetime import timedelta
 
 from discordbot.models import AmongUsGame, AMONGUS_PLAYER_COLORS, AMONGUS_EMOJI_COLORS, VierGewinntGame, VIERGEWINNT_NUMBER_EMOJIS
 from discordbot.botmodules.serverdata import DjangoConnection
+from discordbot.botmodules import apis
 
 import requests
 import os
 import asyncio
+import typing
 
 #####
 
@@ -33,6 +35,8 @@ NO = '❌'
 
 DELETE = '❌'
 
+RUNNING = '⏰'
+
 AMONGUS_TRACKER_INSTALL = """Konfiguriere den Tracker mit folgenden Daten:
                 
 Der Tracker kann [hier](https://raw.githubusercontent.com/rafaelurben/django-discordbot/master/discordbot/files/amongus/tracker.py) heruntergeladen werden. Er funktioniert vorerst nur für Windows uns ist (noch) buggy!
@@ -43,13 +47,6 @@ amongus_last_update = timezone.now()-timedelta(days=1)
 
 #####
 
-@sync_to_async
-def getUpdatedAmongUsGames():
-    global amongus_last_update
-    games = AmongUsGame.objects.filter(last_edited__gt=amongus_last_update)
-    amongus_last_update = timezone.now()
-    return list(games)
-
 async def getAmongUsCategory(guild):
     category = utils.get(guild.categories, name="AmongUs")
     if not category:
@@ -84,25 +81,23 @@ class Games(commands.Cog):
                 except:
                     pass
 
-                await message.remove_reaction(emoji, payload.member)
-
                 game = await DjangoConnection._getAmongUsGame(text_message_id=payload.message_id)
 
                 if emoji == DELETE:
-                    await DjangoConnection._removeAmongUsUser(game=game, userid=payload.user_id, save=True)
+                    await game.remove_user(userid=payload.user_id)
                 else:
                     c = AMONGUS_EMOJI_COLORS[payload.emoji.name]
-                    await DjangoConnection._setAmongUsUser(game=game, userid=payload.user_id, color=c, save=True)
+                    await game.set_user(userid=payload.user_id, color=c)
 
             ### VierGewinnt
 
-            if (emoji in VIERGEWINNT_NUMBER_EMOJIS) and await DjangoConnection._hasVierGewinntGame(message_id=payload.message_id):
+            if (emoji in VIERGEWINNT_NUMBER_EMOJIS) and await DjangoConnection._has(VierGewinntGame, message_id=payload.message_id):
                 try:
                     await message.remove_reaction(emoji, payload.member)
                 except:
                     pass
 
-                game = await DjangoConnection._getVierGewinntGame(message_id=payload.message_id)
+                game = await DjangoConnection._get(VierGewinntGame, message_id=payload.message_id)
 
                 if not game.finished:
                     n = VIERGEWINNT_NUMBER_EMOJIS.index(emoji)
@@ -152,10 +147,9 @@ class Games(commands.Cog):
         aliases=['shop']
     )
     async def fortnite_store(self, ctx):
-        JSON = ctx.apis.Fortnite.getStore()
+        JSON = apis.Fortnite.getStore()
         await ctx.sendEmbed(
             title="Fortnite Item Shop",
-            color=self.color,
             authorurl="http://fortnitetracker.com/",
             authorname="Powered by Fortnitetracker"
         )
@@ -163,7 +157,6 @@ class Games(commands.Cog):
             await ctx.sendEmbed(
                 title=str(JSON[i]["name"]),
                 description=("Rarity: %s \n vBucks: %s" % (JSON[i]["rarity"],JSON[i]["vBucks"])),
-                color=self.color,
                 thumbnailurl=str(JSON[i]["imageUrl"]),
                 footertext="",
                 footerurl="",
@@ -175,10 +168,9 @@ class Games(commands.Cog):
         aliases=[]
     )
     async def fortnite_challenges(self, ctx):
-        JSON = ctx.apis.Fortnite.getChallenges()
+        JSON = apis.Fortnite.getChallenges()
         await ctx.sendEmbed(
             title="Fortnite Challenges",
-            color=self.color,
             fields=[((JSON[i]["metadata"][1]["value"]+" ("+JSON[i]["metadata"][3]["value"]+")"),(JSON[i]["metadata"][5]["value"]+" Battlepassstars")) for i in range(len(JSON))],
             thumbnailurl=str(JSON[0]["metadata"][4]["value"]),
             authorurl="http://fortnitetracker.com/",
@@ -194,12 +186,11 @@ class Games(commands.Cog):
         usage="<Plattform> <Spielername>",
     )
     async def fortnite_stats(self, ctx, platform:str, playername:str):
-        JSON = ctx.apis.Fortnite.getStats(platform, playername)
+        JSON = apis.Fortnite.getStats(platform, playername)
         try:
             await ctx.sendEmbed(
                 title="Fortnite Stats von "+JSON["epicUserHandle"]+" auf "+JSON["platformNameLong"],
                 description=("Account Id: "+JSON["accountId"]),
-                color=self.color,
                 fields=[(JSON["lifeTimeStats"][i]["key"], JSON["lifeTimeStats"][i]["value"]) for i in range(len(JSON["lifeTimeStats"]))],
                 authorurl="http://fortnitetracker.com/",
                 authorname="Powered by Fortnitetracker"
@@ -227,10 +218,10 @@ class Games(commands.Cog):
         usage="<Spielername>",
     )
     async def minecraft_uuid(self, ctx, name):
-        api = ctx.apis.Minecraft
+        api = apis.Minecraft
 
         JSON = api.getProfile(name)
-        EMBED = ctx.getEmbed(title="Minecraft UUID", color=self.color, fields=[("UUID", JSON["id"], False),("Aktueller Name", JSON["name"], False)])
+        EMBED = ctx.getEmbed(title="Minecraft UUID", fields=[("UUID", JSON["id"], False),("Aktueller Name", JSON["name"], False)])
         if "legacy" in JSON:
             EMBED.add_field(name="Account",value="Alter Account")
         if "demo" in JSON:
@@ -244,13 +235,13 @@ class Games(commands.Cog):
         usage="<UUID>",
     )
     async def minecraft_names(self, ctx, uuid):
-        api = ctx.apis.Minecraft
+        api = apis.Minecraft
 
         if len(uuid) != 32:
             raise commands.BadArgument("Eine UUID ist genau 32 Zeichen lang!")
 
         JSON = api.getProfiles(uuid)
-        EMBED = ctx.getEmbed(title="Minecraft Namen", color=self.color, description="Sortierung: Von neu bis alt.")
+        EMBED = ctx.getEmbed(title="Minecraft Namen", description="Sortierung: Von neu bis alt.")
         for i in JSON[::-1]:
             if "changedToAt" in i:
                 EMBED.add_field(name="Name seit "+str(i["changedToAt"]),value=i["name"], inline=False)
@@ -265,13 +256,13 @@ class Games(commands.Cog):
         usage="<UUID>",
     )
     async def minecraft_skin(self, ctx, uuid):
-        api = ctx.apis.Minecraft
+        api = apis.Minecraft
 
         if len(uuid) != 32:
             raise commands.BadArgument("Eine UUID ist genau 32 Zeichen lang!")
 
         JSON = api.getSkin(uuid)
-        EMBED = ctx.getEmbed(title="Minecraft Skin", color=self.color, fields=[("Aktueller Name", JSON["name"]), ("UUID", JSON["id"])])
+        EMBED = ctx.getEmbed(title="Minecraft Skin", fields=[("Aktueller Name", JSON["name"]), ("UUID", JSON["id"])])
         if JSON["skin"] is not None:
             EMBED.set_thumbnail(url=JSON["skin"])
         else:
@@ -286,11 +277,11 @@ class Games(commands.Cog):
         usage="<Spielername>",
     )
     async def minecraft_player(self, ctx, name):
-        api = ctx.apis.Minecraft
+        api = apis.Minecraft
         
         JSON = api.getProfile(name)
         UUID = JSON["id"]
-        EMBED = ctx.getEmbed(title="Minecraft Spieler", color=self.color, fields=[("UUID", UUID)], inline=False)
+        EMBED = ctx.getEmbed(title="Minecraft Spieler", fields=[("UUID", UUID)], inline=False)
         if "legacy" in JSON:
             EMBED.add_field(name="Account",value="Alter Account")
         if "demo" in JSON:
@@ -318,7 +309,11 @@ class Games(commands.Cog):
         def log(*args):
             print('[AmongUs Background Tasks] -', *args)
 
-        for game in await getUpdatedAmongUsGames():
+        global amongus_last_update
+        games = await DjangoConnection._list(AmongUsGame, last_edited__gt=amongus_last_update)
+        amongus_last_update = timezone.now()
+
+        for game in games:
             try:
                 log(f'Updating AmongUs #{ game.pk }...')
 
@@ -433,7 +428,7 @@ class Games(commands.Cog):
                         mention = None
 
                     description = AMONGUS_PLAYER_COLORS[c][2] + " " + c.upper() + " "
-                    description += ("- " + (mention if mention else "") + (" ("+i["name"]+")" if i["name"] else "")) if mention or i["name"] else ""
+                    description += ("- " + (mention if mention else "") + (" ('"+i["name"]+"')" if i["name"] else "")) if mention or i["name"] else ""
 
                     if i["exists"]:
                         if i["alive"]:
@@ -460,20 +455,17 @@ class Games(commands.Cog):
                         In meeting - { inmeeting }
 
                         Spieler:
-
-                        {ALIVE} [Lebend]
-
-                        { t_alive }
-
-                        {DEAD} [Tot]
-
-                        { t_dead }
-
-                        {UNKNOWN} [Unbekannt]
-
-                        { t_unknown }
-                    """),
+                    \u200b"""),
                     color=0xFEDE29,
+                    inline=False,
+                    fields=[
+                        (f"{ALIVE} Lebend",      
+                         '\u200b\n'+t_alive+'\n\u200b'),
+                        (f"{DEAD} Tot",
+                         '\u200b\n'+t_dead+'\n\u200b'),
+                        (f"{UNKNOWN} Unbekannt",  
+                         '\u200b\n'+t_unknown+'\n\u200b'),
+                    ]
                 )
                 await msg.edit(embed=embed)
             except Exception as e:
@@ -493,6 +485,7 @@ class Games(commands.Cog):
         aliases=['au'],
         usage="<Unterbefehl> [Argumente]"
     )
+    @commands.guild_only()
     async def amongus(self, ctx):
         if ctx.invoked_subcommand is None:
             await ctx.send_help()
@@ -620,7 +613,6 @@ class Games(commands.Cog):
         help="Um eine Liste aller VierGewinnt-Befehle zu erhalten, gib den Command ohne Argumente ein.",
         usage="<Unterbefehl> [Argumente]"
     )
-    @commands.guild_only()
     async def viergewinnt(self, ctx):
         if ctx.invoked_subcommand is None:
             await ctx.send_help()
@@ -632,25 +624,28 @@ class Games(commands.Cog):
         usage="<Mitglied> [Breite (4-10)] [Höhe (4-14)]",
     )
     @commands.guild_only()
-    async def viergewinnt_duell(self, ctx, user:User, width:int=7, height:int=6):
-        msg = await ctx.sendEmbed(
-            title="Vier Gewinnt",
-            color=0x0078D7,
-            description=f"Duell gegen {user.mention} wird erstellt..."
-        )
+    async def viergewinnt_duell(self, ctx, user: typing.Union[User, Member], width: int = 7, height: int = 6):
+        if not (user == ctx.author or user.bot):
+            msg = await ctx.sendEmbed(
+                title="Vier Gewinnt",
+                color=0x0078D7,
+                description=f"Duell gegen {user.mention} wird erstellt..."
+            )
 
-        game = await ctx.database._createVierGewinntGame(channel_id=str(ctx.channel.id), message_id=str(msg.id), player_1_id=str(ctx.author.id), player_2_id=str(user.id), width=width, height=height)
+            game = await ctx.database._create(VierGewinntGame, channel_id=str(ctx.channel.id), message_id=str(msg.id), player_1_id=str(ctx.author.id), player_2_id=str(user.id), width=width, height=height)
 
-        embed = ctx.bot.getEmbed(
-            title=f"Vier Gewinnt (#{game.pk})",
-            color=0x0078D7,
-            description=game.get_description()
-        )
+            embed = ctx.bot.getEmbed(
+                title=f"Vier Gewinnt (#{game.pk})",
+                color=0x0078D7,
+                description=game.get_description()
+            )
 
-        await msg.edit(embed=embed)
+            await msg.edit(embed=embed)
 
-        for emoji in VIERGEWINNT_NUMBER_EMOJIS[:game.width]:
-            await msg.add_reaction(emoji)
+            for emoji in VIERGEWINNT_NUMBER_EMOJIS[:game.width]:
+                await msg.add_reaction(emoji)
+        else:
+            raise commands.BadArgument("Du kannst nicht gegen dich selbst oder Bots spielen... Wenn du einen Bot herausfordern möchtest, benutze bitte `/viergewinnt challenge`!")
 
     @viergewinnt.command(
         name="challenge",
@@ -665,7 +660,7 @@ class Games(commands.Cog):
             description=f"Challenge gegen einen Bot wird erstellt..."
         )
 
-        game = await ctx.database._createVierGewinntGame(channel_id=str(ctx.channel.id), message_id=str(msg.id), player_1_id=str(ctx.author.id), player_2_id=None, width=width, height=height)
+        game = await ctx.database._create(VierGewinntGame, channel_id=str(ctx.channel.id), message_id=str(msg.id), player_1_id=str(ctx.author.id), player_2_id=None, width=width, height=height)
 
         embed = ctx.bot.getEmbed(
             title=f"Vier Gewinnt (#{game.pk})",
@@ -678,5 +673,111 @@ class Games(commands.Cog):
         for emoji in VIERGEWINNT_NUMBER_EMOJIS[:game.width]:
             await msg.add_reaction(emoji)
 
+    @viergewinnt.command(
+        name="games",
+        brief="Liste alle deine Spiele auf",
+        aliases=["list"],
+        usage="[User]",
+    )
+    async def viergewinnt_games(self, ctx, user: typing.Union[User, Member] = None):
+        user = user or ctx.author
+        challenges = await ctx.database._list(VierGewinntGame, _order_by=("-time_created",), player_1_id=str(user.id), player_2_id__isnull=True)
+        duels_created = await ctx.database._list(VierGewinntGame, _order_by=("-time_created",), player_1_id=str(user.id), player_2_id__isnull=False)
+        duels_invited = await ctx.database._list(VierGewinntGame, _order_by=("-time_created",), player_2_id=str(user.id))
+        challengetext = "" if challenges else "Du hast noch keine Challenges erstellt."
+        for g in challenges:
+            challengetext += ((UNKNOWN if g.winner_id is None else YES if g.winner_id == str(user.id) else NO)
+                              if g.finished else RUNNING) + f" ({ g.id }) BOT " + g.time_created.strftime("%Y/%m/%d - %H:%M %Z") + "\n"
+        duelscreatedtext = "" if duels_created else "Du hast noch keine Duelle erstellt."
+        for g in duels_created:
+            duelscreatedtext += ((UNKNOWN if g.winner_id is None else YES if g.winner_id == str(user.id) else NO) 
+                                 if g.finished else RUNNING) + f" ({ g.id }) <@{ g.player_2_id }> " + g.time_created.strftime("%Y/%m/%d - %H:%M %Z") + "\n"
+        duelsinvitedtext = "" if duels_invited else "Du wurdest noch nicht zu einem Duell eingeladen."
+        for g in duels_invited:
+            duelsinvitedtext += ((UNKNOWN if g.winner_id is None else YES if g.winner_id == str(user.id) else NO)
+                                 if g.finished else RUNNING) + f" ({ g.id }) <@{ g.player_1_id }> " + g.time_created.strftime("%Y/%m/%d - %H:%M %Z") + "\n"
+
+        await ctx.sendEmbed(
+            title="VierGewinnt Spiele", 
+            description="Vier Gewinnt Spiele von "+user.mention, 
+            color=0x0078D7, 
+            inline=False,
+            fields=[
+                ("Challenges", challengetext+"\u200b"), 
+                ("Erstellte Spiele", duelscreatedtext+"\u200b"), 
+                ("Eingeladene Spiele", duelsinvitedtext+"\u200b")
+            ]
+        )
+
+    @viergewinnt.command(
+        name="reset",
+        brief="Lösche alle deine Challenges und Duelle",
+        description="Hinweis: Dies beinhaltet nicht Duelle, welche ein anderer Spieler gestartet hat!",
+        aliases=[],
+        usage="[NUR BESITZER: Spieler]"
+    )
+    async def viergewinnt_reset(self, ctx, user: typing.Union[User, Member] = None):
+        user = ctx.author if user is None else user if await self.bot.is_owner(user) else ctx.author
+        await ctx.database._delete(await ctx.database._list(VierGewinntGame, get_as_queryset=True, player_1_id=str(user.id)))
+        await ctx.sendEmbed(
+            title="VierGewinnt Spiele gelöscht!",
+            description=f"Die VierGewinnt Spiele von { user.mention } wurden erfolgreich gelöscht!",
+            color=0x0078D7, 
+        )
+
+    @viergewinnt.command(
+        name="resume",
+        brief="Fahre ein Duell oder eine Challenge fort",
+        description="Wenn du die Nachricht eines Duells oder einer Challenge nicht mehr finden kannst, kannst du sie mit diesem Befehl noch einmal senden lassen. Hiermit können auch Spiele von anderen Spielern angesehen werden.",
+        aliases=["continue", "wiederaufnehmen", "resend", "view"],
+        usage="<ID>",
+        help="Um die ID herauszufinden, benutze `/viergewinnt games`",
+    )
+    async def viergewinnt_resume(self, ctx, id: int):
+        if await ctx.database._has(VierGewinntGame, id=id):
+            game = await ctx.database._get(VierGewinntGame, id=id)
+
+            if str(ctx.author.id) in [game.player_1_id, game.player_2_id]:
+                msg = await ctx.sendEmbed(
+                    title=f"Vier Gewinnt (#{game.pk})",
+                    color=0x0078D7,
+                    description=game.get_description()
+                )
+
+                oldchannelid = int(game.channel_id)
+                oldmessageid = int(game.message_id)
+
+                game.channel_id = msg.channel.id
+                game.message_id = msg.id
+
+                await ctx.database._save(game)
+
+                if not game.finished:
+                    for emoji in VIERGEWINNT_NUMBER_EMOJIS[:game.width]:
+                        await msg.add_reaction(emoji)
+
+                    try:
+                        channel = self.bot.get_channel(oldchannelid) or await self.bot.fetch_channel(oldchannelid)
+                        message = await channel.fetch_message(oldmessageid)
+
+                        await message.edit(embed=self.bot.getEmbed(
+                            title=f"Vier Gewinnt (#{game.pk})",
+                            color=0x0078D7,
+                            description="Dieses Spiel wurde in einer neuen Nachricht weitergeführt!\n\n"+game.get_description()
+                        ))
+
+                        for emoji in VIERGEWINNT_NUMBER_EMOJIS[:game.width]:
+                            await message.remove_reaction(emoji, self.bot.user)
+                    except NotFound as e:
+                        print(e)
+            else:
+                msg = await ctx.sendEmbed(
+                    title=f"Vier Gewinnt (#{game.pk} - Readonly)",
+                    color=0x0078D7,
+                    description=game.get_description()
+                )
+        else:
+            raise commands.BadArgument("Ein Spiel mit dieser ID konnte nicht gefunden werden! Möglicherweise gelöscht?")
+
 def setup(bot):
     bot.add_cog(Games(bot))
