@@ -1,21 +1,19 @@
 # pylint: disable=no-member, unsubscriptable-object
 
 from django.db import models
-from django.conf import settings
 from django.urls import reverse
 from django.utils import timezone
 
 from discordbot.botmodules.bots import VierGewinntBot
+from discordbot.botmodules.parser import HTMLCleaner
+
+from discordbot.config import DOMAIN
+
+from asgiref.sync import sync_to_async
 
 import time
 import uuid
-
-# Helper functions
-
-
-def get_discordbot_domain():
-    return getattr(settings, "DISCORDBOT_DOMAIN", None)
-
+import re
 
 # Basic
 
@@ -38,35 +36,44 @@ class Server(models.Model):
     name = models.CharField("Name", default="", blank=True, max_length=100)
     members = models.ManyToManyField("User", through="Member")
 
+    playlist = models.ForeignKey("Playlist", on_delete=models.SET_NULL, default=None, blank=True, null=True, related_name="+")
+
+    @sync_to_async
+    def getPlaylist(self):
+        if self.playlist is None:
+            self.playlist = Playlist.objects.create(server=self, title="Playlist in "+self.name)
+            self.save()
+        return self.playlist
+
     def reportCount(self):
         return self.reports.count()
     reportCount.short_description = "Reports"
 
+    @sync_to_async
     def getReports(self, user=None):
         if user is None:
-            reports = []
+            description = ""
             for member in self.members.all():
-                count = member.reportCount()
-                if count > 0:
-                    reports.append({
-                        "name": str(count)+" Report(s)",
-                        "value": member.mention,
-                        "inline": False
-                    })
-            return reports
+                count = int(member.reportCount(server=self))
+                if count == 1:
+                    description += f"{member.mention} (Ein Report)\n"
+                elif count > 1:
+                    description += f"{member.mention} ({count} Reports)\n"
+            return description
         else:
             return user.getReports(server=self)
 
     def memberCount(self):
         return self.members.count()
+    memberCount.short_description = "Members"
 
     def __str__(self):
         return self.name+" ("+str(self.id)+")"
-    __str__.short_description = "Server"
+    __str__.short_description = "Guild"
 
     class Meta():
-        verbose_name = "Server"
-        verbose_name_plural = "Server"
+        verbose_name = "Guild"
+        verbose_name_plural = "Guilds"
 
     objects = models.Manager()
 
@@ -74,27 +81,25 @@ class User(models.Model):
     id = models.CharField("Discord ID", primary_key=True, max_length=20)
     name = models.CharField("Name", default="", blank=True, max_length=100)
 
-    def reportCount(self):
-        return self.reports.count()
+    def reportCount(self, **filters):
+        return self.reports.filter(**filters).count()
     reportCount.short_description = "Reports"
 
-    def createdReportCount(self):
-        return self.created_reports.count()
-    createdReportCount.short_description = "Erstellte Reports"
+    def createdReportCount(self, **filters):
+        return self.created_reports.filter(**filters).count()
+    createdReportCount.short_description = "Created reports"
 
-    def serverCount(self):
-        return self.servers.count()
-    serverCount.short_description = "Anzahl Server"
+    def serverCount(self, **filters):
+        return self.servers.filter(**filters).count()
+    serverCount.short_description = "Server count"
 
+    @sync_to_async
     def joinServer(self, server):
         if not server.members.filter(id=self.id).exists():
             server.members.add(self)
 
-    def getReports(self, server=None):
-        if server is None:
-            reports = self.reports.all()
-        else:
-            reports = self.reports.filter(server=server)
+    def getReports(self, **filters):
+        reports = self.reports.filter(**filters)
         return [
             report.getEmbedField() for report in reports
         ]
@@ -113,6 +118,146 @@ class User(models.Model):
     
     objects = models.Manager()
 
+# Music
+
+class AudioSource(models.Model):
+    url_source = models.TextField("Url (Source)")
+
+    url_watch = models.URLField("Url (Watch)", default="", blank=True)
+    url_thumbnail = models.URLField("Url (Thumbnail)", default="", blank=True)
+
+    title = models.CharField("Title", max_length=256)
+    description = models.TextField("Description", max_length=2048)
+    duration = models.IntegerField("Duration", default=0, blank=True)
+
+    uploader_name = models.CharField("Uploader name", max_length=256, default="", blank=True)
+    uploader_url = models.URLField("Uploader Url", default="", blank=True)
+
+    @classmethod
+    @sync_to_async
+    def create_from_dict(self, data:dict):
+        # if "formats" in data:
+        #     data.pop("formats")
+        # if "thumbnails" in data:
+        #     data.pop("thumbnails")
+        # print(data)
+
+        if not data is None:
+            url_source = data.get('url', '')
+
+            url_watch = data.get('webpage_url', url_source)[:200]
+            url_thumbnail = data.get('thumbnail', "")[:200]
+
+            title = data.get('title', 'Unbekannter Titel')[:256]
+            description = data.get('description', "Keine Beschreibung gefunden.")[:2048]
+            duration = int(data.get('duration', 0))
+            
+            uploader_name = data.get('uploader', "")[:256]
+            uploader_url = data.get('uploader_url', "")[:200]
+            
+            return self.objects.create(
+                url_source=url_source,
+
+                url_watch=url_watch,
+                url_thumbnail=url_thumbnail,
+                
+                title=title,
+                description=description,
+                duration=duration,
+
+                uploader_name=uploader_name,
+                uploader_url=uploader_url,
+            )
+        else:
+            return None
+
+    @property
+    def clickable(self):
+        return f"[{self.title}]({self.url_watch}) [{self.duration_calc}]"
+
+    @property
+    def duration_calc(self):
+        h = str(self.duration//3600)
+        m = str(self.duration%3600//60)
+        s = str(self.duration%3600%60)
+        return f"{h}:{m}:{s}" if h != "0" else f"{m}:{s}"
+
+    def __str__(self):
+        return f"{self.title} [{self.duration_calc}]"
+    __str__.short_description = "Audio source"
+
+    class Meta():
+        verbose_name = "Audio source"
+        verbose_name_plural = "Audio sources"
+
+    objects = models.Manager()
+
+class PlaylistPosition(models.Model):
+    source = models.ForeignKey('AudioSource', on_delete=models.PROTECT)
+    queue = models.ForeignKey('Playlist', on_delete=models.CASCADE)
+    position = models.PositiveSmallIntegerField("Position")
+
+    class Meta():
+        verbose_name = "Audio queue position"
+        verbose_name_plural = "Audio queue positions"
+
+    objects = models.Manager()
+
+class Playlist(models.Model):
+    server = models.ForeignKey('Server', on_delete=models.CASCADE, related_name="playlists")
+    title = models.CharField("Title", max_length=256, default="Server default")
+
+    sources = models.ManyToManyField('AudioSource', through="PlaylistPosition")
+    current_pos = models.PositiveSmallIntegerField("Current position", default=0)
+
+    @sync_to_async
+    def addSource(self, source):
+        pos = self.sources.count()+1
+        self.sources.add(source, through_defaults={"position": pos})
+        return pos
+
+    @sync_to_async
+    def removePosition(self, pos: int):
+        if pos > 0 and pos <= self.sources.count() and not self.current_pos == pos:
+            self.sources.through.objects.filter(position=pos).delete()
+            for s in self.sources.through.objects.filter(position__gt=pos):
+                s.position -= 1
+                s.save()
+
+    @sync_to_async
+    def switchPositions(self, pos1:int, pos2:int):
+        if self.sources.count() > max(pos1, pos2):
+            o1 = self.sources.through.objects.get(playlist=self, position=pos1)
+            o2 = self.sources.through.objects.get(playlist=self, position=pos2)
+            o1.position = pos2
+            o2.position = pos1
+            o1.save()
+            o2.save()
+            return True
+        return False
+
+    @sync_to_async
+    def getAll(self):
+        return list(self.sources.all().order_by("position"))
+
+    @sync_to_async
+    def hasNext(self):
+        return self.sources.through.objects.filter(playlist=self, position__gt=self.current_pos).exists()
+
+    @sync_to_async
+    def next(self):
+        self.current_pos += 1
+        obj = self.sources.through.objects.get(playlist=self, position=self.current_pos)
+        self.save()
+        return obj
+
+    class Meta():
+        verbose_name = "Audio queue"
+        verbose_name_plural = "Audio queues"
+
+    objects = models.Manager()
+
+
 # Support
 
 class Report(models.Model):
@@ -124,11 +269,11 @@ class Report(models.Model):
     timestamp = models.DateTimeField("Zeitpunkt", auto_now_add=True)
 
     def getEmbedField(self):
-        return {
-            "name": str(self.timestamp.strftime('%d.%m.%Y - %H:%M:%S')),
-            "value": str(self.reason)+" - "+self.reported_by.mention,
-            "inline": False
-        }
+        return (
+            f"{self.timestamp.strftime('%Y/%m/%d %H:%M:%S')} ({self.pk})",
+            f"{self.reported_by.mention} - {self.reason}",
+            False
+        )
 
     def __str__(self):
         return "Report #"+str(self.pk)
@@ -230,9 +375,8 @@ class AmongUsGame(models.Model):
 
     
     def get_tracker_url(self):
-        domain = get_discordbot_domain()
-        if domain:
-            return domain+reverse("discordbot:amongus-tracker-post")
+        if DOMAIN:
+            return DOMAIN+reverse("discordbot:amongus-tracker-post")
         else:
             return "Frage den Bot-Owner!"
 
@@ -295,21 +439,20 @@ class AmongUsGame(models.Model):
 
     # User
 
-    def remove_user(self, userid, save=False):
+    @sync_to_async
+    def remove_user(self, userid):
         for c in AMONGUS_PLAYER_COLORS:
             if getattr(self, f'p_{c}_userid') == str(userid):
                 setattr(self, f'p_{c}_userid', "")
+        self.save()
 
-        if save:
-            self.save()
-
-    def set_user(self, userid, color, save=False):
-        self.remove_user(userid)
+    @sync_to_async
+    def set_user(self, userid, color):
+        for c in AMONGUS_PLAYER_COLORS:
+            if getattr(self, f'p_{c}_userid') == str(userid):
+                setattr(self, f'p_{c}_userid', "")
         setattr(self, f'p_{color}_userid', str(userid))
-
-        if save:
-            self.save()
-
+        self.save()
 
     def __str__(self):
         return "AmongUs #"+str(self.pk)
@@ -399,6 +542,7 @@ class VierGewinntGame(models.Model):
 
     def get_description(self):
         return self._get_game_info()+"\n\n"+self._get_players()+"\n\n"+self._get_gameboard()
+    get_description.short_description = "Game"
 
     # Get
 
@@ -514,3 +658,60 @@ class VierGewinntGame(models.Model):
         verbose_name_plural = 'VierGewinnt Games'
 
     objects = models.Manager()
+
+# Support
+
+## BotPermission (for Remote)
+
+class BotPermission(models.Model):
+    id_1 = models.CharField("ID 1", max_length=32)
+    id_2 = models.CharField("ID 2", max_length=32)
+    typ = models.CharField("Typ", max_length=32)
+
+    objects = models.Manager()
+
+# Notifier
+
+NOTIFIER_WHERE_TYPES = [
+    ("channel", "Kanal"),
+    ("member",  "Mitglied (DM)"),
+]
+
+NOTIFIER_FREQUENCIES = [
+    ("hour",    "Stündlich"),
+    ("minute",  "Minütlich"),
+]
+
+class NotifierSub(models.Model):
+    name = models.CharField("Name", default="Unbenannte Mitteilung", max_length=50)
+
+    where_type = models.CharField("Wohin: Typ", max_length=8, choices=NOTIFIER_WHERE_TYPES)
+    where_id = models.CharField("Wohin: ID", max_length=32)
+
+    frequency = models.CharField("Häufigkeit", max_length=8, choices=NOTIFIER_FREQUENCIES)
+
+    url = models.URLField("URL")
+
+    must_contain_regex = models.CharField("Muss Regex enthalten", default="", blank=True, max_length=32)
+
+    last_hash = models.CharField("Letzter Hash", max_length=32, editable=False)
+
+    def process(self):
+        data = HTMLCleaner.from_url(self.url)
+        thishash = str(hash(data))
+        if thishash != str(self.last_hash):
+            self.last_hash = thishash
+            if (not self.must_contain_regex) or re.search(self.must_contain_regex, data):
+                return data
+            return True
+        return False
+
+    def __str__(self):
+        return self.name
+    __str__.short_description = "NotifierSub"
+
+    class Meta:
+        verbose_name = 'NotifierSub'
+        verbose_name_plural = 'NotifierSubs'
+
+    objects = models.Manager()
